# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""A Token is an occurence of a term from the text of a field.  It consists of
  a term's text, the start and end offset of the term in the text of the field,
  and a type string.

  The start and end offsets permit applications to re-associate a token with
  its source text, e.g., to display highlighted query terms in a document
  browser, or to show matching text fragments in a KWIC (KeyWord In Context)
  display, etc.

  The type is an interned string, assigned by a lexical analyzer
  (a.k.a. tokenizer), naming the lexical or syntactic class that the token
  belongs to.  For example an end of sentence marker token might be implemented
  with type "eos".  The default token type is "word".
  """


class Token(object):
    __slots__ = ['endOffsett', 'startOffsett', 'termTextt', 'typpe']


    def __init__(self, text, start, end, typ='word'):
        self.termTextt = text
        self.startOffsett = start
        self.endOffsett = end
        self.typpe = typ


    def endOffset(self):
        return self.endOffsett

    def startOffset(self):
        return self.startOffsett

    def type(self):
        # NB conflict with built-in function type()?
        return self.typpe

    def termText(self):
        return self.termTextt
