# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""Documents are the unit of indexing and search.

A Document is a set of fields.  Each field has a name and a textual
value.  A field may be stored with the document, in which case it is
returned with search hits on the document.  Thus each document should
typically contain stored fields which uniquely identify it.
"""


class Document(object):

    def __init__(self):
        self._fields = {}
        self.fieldNames = []
    
    def add(self, field):
        """Adds a field to a document."""
        name = field.name()
        self._fields[name] = field
        if name not in self.fieldNames:
            self.fieldNames.append(name)
    
    def getField(self, name):
        """Returns a field with the given name, or None if none exist."""
        return self._fields.get(name, None)

    def get(self, name):
        """Returns the string value of a field, or None."""
        field = self.getField(name)
        if field is not None:
            return field.stringValue()
        else:
            return None

    def fields(self):
        """Return Python iterator over fields."""
        return [self._fields[name] for name in self.fieldNames]

    def __repr__(self):
        return '<Document[%s]>' % ("|".join(self.fieldNames),)
