# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import fieldinfo


class FieldInfos(object):
        
    def __init__(self, d=None, name=None):
        self.byNumber = []
        self.byName = {}
        if d is None and name is None:
            self.addString('',False)
        else:
            input = d.openFile(name)
            try:
                self.read(input)
            finally:
                input.close()

    def add(self, doc):
        """Adds field info for a Document"""
        for field in doc.fields():
            self.addString(field.name(), field.isIndexed)

    def addString(self, name, isIndxd):
        fi = self.fieldInfo(name)
        if fi is None:
            self.addInternal(name, isIndxd)
        elif fi.isIndexed is not isIndxd:
            fi.isIndexed = True

    def addFieldInfos(self, other):
        """Merges in information from another FieldInfos"""
        for i in range(len(other)):
            fi = other.fieldInfoInt(i)
            self.addString(fi.name, fi.isIndexed)

    def addInternal(self, name, isIndexed):
        fi = fieldinfo.FieldInfo(name, isIndexed, len(self.byNumber))

        self.byNumber.append(fi)
        self.byName[name]=fi

    def fieldNumber(self, fieldName):
        fi = self.fieldInfo(fieldName)
        if fi is not None:
            return fi.number
        else:
            return -1

    def fieldInfo(self, fieldName):
        return self.byName.get(fieldName, None)

    def fieldName(self, fieldNumber):
        return self.byNumber[fieldNumber].name

    def fieldInfoInt(self, fieldNumber):
        return self.byNumber[fieldNumber]

    def __len__(self):
        return len(self.byNumber)

    def writeDir(self, d, name):
        output = d.createFile(name)
        try:
            self.write(output)
        finally:
            output.close()

    def write(self, output):
        output.writeVInt(len(self))

        for i in range(len(self)):
            fi = self.fieldInfoInt(i)
            output.writeString(fi.name)
            if fi.isIndexed:
                output.writeByte(1)
            else:
                output.writeByte(0)

    def read(self, input):
        size = input.readVInt()
        for i in range(size):
            self.addInternal(input.readString(), (input.readByte() != 0))

    def fieldNames(self):
        # Experimental for auto-queries
        return self.byName.keys()
