# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import lupy.document.document as document
import lupy.document.field as field

class FieldsReader(object):
    __slots__ = ['fieldInfos', 'fieldsStream', 'indexStream', 'sze']


    def __init__(self, d, segment, fn):
        self.fieldInfos = fn

        self.fieldsStream = d.openFile(segment + '.fdt')
        self.indexStream = d.openFile(segment + '.fdx')
                                      
        self.sze = self.indexStream.length / 8


    def close(self):
        self.fieldsStream.close()
        self.indexStream.close()


    def size(self):
        return self.sze


    def doc(self, n):
        self.indexStream.seek(n * 8L)
        position = self.indexStream.readLong()
        self.fieldsStream.seek(position)

        doc = document.Document()
        numFields = self.fieldsStream.readVInt()
        for i in range(numFields):
            fieldNumber = self.fieldsStream.readVInt()
            fi = self.fieldInfos.fieldInfoInt(fieldNumber)

            bits = self.fieldsStream.readByte()
            tokenized = ((bits & 1) != 0)

            doc.add(field.Field(fi.name, self.fieldsStream.readString(),
                          True, fi.isIndexed, tokenized))

        return doc

    
