# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

class FieldsWriter(object):
    __slots__ = ['fieldInfos', 'fieldsStream', 'indexStream']

    def __init__(self, d, segment, fn):
        self.fieldInfos = fn
        self.fieldsStream = d.createFile(segment + '.fdt')
        self.indexStream = d.createFile(segment + '.fdx')


    def addDocument(self, doc):
        self.indexStream.writeLong(self.fieldsStream.getFilePointer())
        storedCount = 0
        for field in doc.fields():
            if field.isStored:
                storedCount += 1

        self.fieldsStream.writeVInt(storedCount)

        for field in doc.fields():
            if field.isStored:
                self.fieldsStream.writeVInt(self.fieldInfos.fieldNumber(field.name()))
    
                bits = 0
                if field.isTokenized:
                    bits |= 1
                self.fieldsStream.writeByte(bits)
    
                self.fieldsStream.writeString(field.stringValue())


    def close(self):
        self.fieldsStream.close()
        self.indexStream.close()

                                           
