# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import segmentinfo

class SegmentInfos(list):
    __slots__ = ['counter']

    def __init__(self, list = None): #name, docCount, directory)
        self.counter = 0
        if list is not None:
            for i in list:
                self.append(i)
    
    def __getslice__(self, lo, hi):
        res = SegmentInfos(list.__getslice__(self, lo, hi))
        res.counter = self.counter
        return res
    
    def read(self, directory):
        input = directory.openFile('segments')
        try:
            self.counter = input.readInt()      # read counter
            i = input.readInt()
            while i > 0:                        # read segment infos
                si = segmentinfo.SegmentInfo(input.readString(),
                                             input.readInt(),
                                             directory)
                self.append(si)
                i -= 1
        finally:
            input.close()

    def write(self, directory):
        output = directory.createFile('segments.new')
        try:
            output.writeInt(self.counter)
            output.writeInt(len(self))
            for si in self:
                output.writeString(si.name)
                output.writeInt(si.docCount)
        finally:
            output.close()

        # Install new segment info
        directory.renameFile('segments.new','segments')
        
    def __repr__(self):
        return 'SegInfo' + list.__repr__(self)
        
