# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import copy

class SegmentTermDocs(object):
    __slots__ = ['deletedDocs', 'docu', 'frq', 'freqCount',
                 'freqStream', 'parent']


    
    def __init__(self, parent):
        self.parent = parent
        self.freqStream = parent.freqStream.clone()
        self.deletedDocs = parent.deletedDocs

        self.docu = 0


    def close(self):
        self.freqStream.close()


    def doc(self):
        return self.docu


    def freq(self):
        return self.frq


    def next(self):
        while True:
            if self.freqCount == 0:
                return False

            docCode = self.freqStream.readVInt()
            # shift off low bit
            # uses >>>
            self.docu += docCode >> 1
            if (docCode & 1) != 0:
                self.frq = 1
            else:
                self.frq = self.freqStream.readVInt()

            self.freqCount -= 1

            if self.deletedDocs is None or (not self.deletedDocs.get(self.docu)):
                break

            self.skippingDoc()

        return True



    def read(self, docs, freqs):
        end = len(docs)
        i = 0
        while i < end and self.freqCount > 0:
            # manually inlined call to next() for speed
            docCode = self.freqStream.readVInt()
            # shift off low bit
            # uses >>>
            self.docu += docCode >> 1
            if docCode & 1 != 0:
                # if low bit is set
                # freq is 1
                self.frq = 1
            else:
                # else read freq
                self.frq = self.freqStream.readVInt()

            self.freqCount -= 1

            if self.deletedDocs is None or not self.deletedDocs.get(self.docu):
                docs[i] = self.docu
                freqs[i] = self.frq
                i += 1

        return i


    def seekTerm(self, term):
        ti = self.parent.tis.getTerm(term)
        self.seekTi(ti)


    def seekTi(self, ti):
        if ti is None:
            self.freqCount = 0
        else:
            self.freqCount = ti.docFreq
            self.docu = 0
            self.freqStream.seek(ti.freqPointer)
            

    def skippingDoc(self):
        '''Do nothing'''
        

    def skipTo(self, target):
        while target > self.docu:
            if not self.next():
                return False

        return True
