# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

from termenum import TermEnum
import term
import terminfo
import copy

class SegmentTermEnum(TermEnum):
##    __slots__ = ['buffer', 'fileInfos', 'indexPointer', 'input',
##                 'isIndex', 'position', 'prev', 'size', 'term', 'trmInfo']


    def __init__(self, i, fis, isi):
        self.input = i
        self.fieldInfos = fis
        self.size = self.input.readInt()
        self.isIndex = isi

        self.indexPointer = 0
        self.position = -1
        self.buffer = []
        self.prev = None

        self.term = term.Term('','')
        self.trmInfo = terminfo.TermInfo()
    
    
    def clone(self):
        """Return a copy of self.
        """
        
        # TODO: implement as __copy__
        clone = copy.copy(self)
        clone.input = self.input.clone()
       
        clone.trmInfo = terminfo.TermInfo()
        clone.trmInfo.setTo(self.trmInfo)

        if self.term is not None:
            clone.growBuffer(len(self.term.text()))

        return clone


    def close(self):
        self.input.close()


    def docFreq(self):
        return self.trmInfo.docFreq


    def freqPointer(self):
        return self.trmInfo.freqPointer


    def growBuffer(self, length):
        self.buffer = [0] * length
        for i in range(len(self.term.text())):
            self.buffer[i] = self.term.txt[i]


    def next(self):
        self.position += 1
        
        if self.position > self.size -1:
            self.position += 1
            self.term = None
            return False

        #self.position += 1

        self.prev = self.term
        self.term = self.readTerm()

        self.trmInfo.docFreq = self.input.readVInt()
        self.trmInfo.freqPointer += self.input.readVLong()
        self.trmInfo.proxPointer += self.input.readVLong()

        if self.isIndex:
            self.indexPointer += self.input.readVLong()

        return True


    def proxPointer(self):
        return self.trmInfo.proxPointer


    def readTerm(self):
        start = self.input.readVInt()
        length = self.input.readVInt()
        totalLength = start + length
        
        # print self.buffer, start, length, totalLength

        if len(self.buffer) < totalLength:
            self.growBuffer(totalLength)

        self.input.readChars(self.buffer, start, length)

        fi = self.input.readVInt()
        fld = self.fieldInfos.fieldName(fi)
        txt = ''.join(self.buffer[0:totalLength])
        t = term.Term(fld,txt,False)
        return t


    def seek(self, pointer, p, t, ti):
        self.input.seek(pointer)
        self.position = p
        self.term = t
        self.prev = None
        self.trmInfo.setTo(ti)
        self.growBuffer(len(self.term.text()))


    #def term(self):
        #return self.term

    def termInfo(self, ti=None):
        if ti is None:
            nti = terminfo.TermInfo()
            nti.setTo(self.trmInfo)
            return nti
        else:
            ti.setTo(self.trmInfo)
