# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

from segmenttermdocs import SegmentTermDocs

class Term(object):
    __slots__ = ['fld', 'txt']


    def __init__(self, fld, txt, intrn = True):
        self.fld = fld
        self.txt = txt


    def __cmp__(self, other):
        """Compares two terms, returning an integer which is less than zero iff this
        term belongs after the argument, equal zero iff this term is equal to the
        argument, and greater than zero iff this term belongs after the argument.

        The ordering of terms is first by field, then by text."""

        if self.fld == other.fld:
            # fields are interned
            return cmp(self.txt, other.txt)
        else:
            return cmp(self.fld, other.fld)

    def __hash__(self):
        return hash(self.fld + self.txt)


    def equals(self, obj):
        """Compares two terms, returning True iff they have the same
        field and text"""

        if obj is None:
            return False
        return field is other.fld and self.txt.equals(other.txt)


    def field(self):
        return self.fld
    

    def hasCode(self):
        return self.fld.hasCode() + self.txt.hashCode()


    def readObject(self, inp):
        inp.defaultReadObject()
        #self.fld = intern(self.fld)


    def set(self, fld, txt):
        self.fld = fld
        self.txt = txt


    def text(self):
        return self.txt


    def __repr__(self):
        return 'Term<'+self.fld+':'+self.txt+'>'

