# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import terminfoswriter as tiw
import segmenttermenum as ste


class TermInfosReader(object):
    __slots__ = ['directory', 'enum', 'fieldInfos', 'indexInfos', 'indexPointers',
                 'indexTerms', 'segment', 'sze']


    def __init__(self, d, seg, fis):
        self.directory = d
        self.segment = seg
        self.fieldInfos = fis
        
        self.indexTerms = None

        self.enum = ste.SegmentTermEnum(
            self.directory.openFile(self.segment + '.tis'),
            self.fieldInfos,
            False)
        
        self.sze = self.enum.size
        self.readIndex()


    def close(self):
        if self.enum is not None:
            self.enum.close()



    def getInt(self, position):
        if self.sze == 0:
            return None

        if (self.enum is not None and self.enum.term() is not None and
            position > self.enum.position and
            position < (self.enum.position + tiw.TermInfosWriter.INDEX_INTERVAL)):
            # can avoid seek
            return self.scanEnum(position)

        # must seek
        self.seekEnum(position/TermInfosWriter.INDEX_INTERVAL)
        return self.scanEnum(position)


    def getIndexOffset(self, term):
        #TODO - user biset module?
  
        lo = 0
        hi = len(self.indexTerms) - 1
        
        while hi >= lo:
            mid = (lo + hi) >> 1
            delta = cmp(term, self.indexTerms[mid])
            if delta < 0:
                hi = mid - 1
            elif delta > 0:
                lo = mid + 1
            else:
                return mid
                
        return hi
    

    def getTerm(self, term):
        if self.sze == 0:
            return None

        # Optimize sequential access: first try scanning
        # cached enum w/o seeking

        if (self.enum.term is not None and
            ((self.enum.prev is not None and cmp(term,self.enum.prev) > 0) or
             cmp(term,self.enum.term) >= 0)):
            # term is at or past current
            enumOffset = (self.enum.position/tiw.TermInfosWriter.INDEX_INTERVAL)+1

            if (len(self.indexTerms) == enumOffset or
                cmp(term, self.indexTerms[enumOffset]) < 0):
                # but before end of block
                # no need to seek
                return self.scanEnum(term)

        # random-access: must seek
        self.seekEnum(self.getIndexOffset(term))
        return self.scanEnum(term)
                                          


    def getPosition(self, term):
        if size == 0:
            reutrn -1

        indexOffset = self.getIndexOffest(term)
        self.seekEnum(indexOffset)

        while (term.compareTo(self.enum.term()) > 0 and self.enum.next()):
            pass

        if term.compareTo(self.enum.term()) == 0:
            return self.enum.position
        else:
            return -1


    def readIndex(self):
        indexEnum = ste.SegmentTermEnum(
            self.directory.openFile(self.segment + '.tii'),
            self.fieldInfos,
            True)

        try:
            indexSize = indexEnum.size

            self.indexTerms = []
            self.indexInfos = []
            self.indexPointers = []

            while indexEnum.next():
                self.indexTerms.append(indexEnum.term)
                self.indexInfos.append(indexEnum.termInfo())
                self.indexPointers.append(indexEnum.indexPointer)

        finally:
            indexEnum.close()


    def scanEnum(self, position):
        while(self.enum.position < position):
            if not enum.next():
                return None
        return self.enum.term()


    def scanEnum(self, term):
        # Scans within block for matching term.
        while (cmp(term, self.enum.term) > 0 and self.enum.next()):
            pass
        if (self.enum.term is not None and cmp(term, self.enum.term) == 0):
            return self.enum.termInfo()
        else:
            return None


    def seekEnum(self, indexOffset):
        self.enum.seek(self.indexPointers[indexOffset],
                       (indexOffset * tiw.TermInfosWriter.INDEX_INTERVAL) - 1,
                       self.indexTerms[indexOffset], self.indexInfos[indexOffset])


    def size(self):
        # Return the number of term/value pairs in this set
        return self.sze


    def terms(self, term = None):
        if term is None:
            # Returns an enumeration of all the Terms and TermInfos in the set
            if (self.enum.position != -1):
                # if not at start
                # reset to start
                self.seekEnum(0)
        else:
            self.getTerm(term)
            
        res = self.enum.clone()
        return res
    
