# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import term
import terminfo


class TermInfosWriter(object):
    __slots__ = ['fieldInfos', 'isIndex', 'lastIndexPointer', 'lastTerm', 'lastTi',
                 'other', 'output', 'size']
    
    
    INDEX_INTERVAL = 128


    def __init__(self, d, seg, fis, isIndex = False):
        
        self.initialize(d, seg, fis, isIndex)
        
        self.size = 0
        self.lastIndexPointer = 0
        self.lastTerm = term.Term('','')
        self.lastTi = terminfo.TermInfo()
        
        if isIndex is False:
            self.other = TermInfosWriter(d, seg, fis, True)
            self.other.other = self

            
    def initialize(self, d, seg, fis, isi):
        self.fieldInfos = fis
        self.isIndex = isi
        if isi is True:
            ext = '.tii'
        else:
            ext = '.tis'
            
        self.output=d.createFile(seg + ext)
        # leave space for size
        self.output.writeInt(0)


    def stringDifference(self, s1, s2):
        len1 = len(s1)
        len2 = len(s2)

        if len1 < len2:
            len3 = len1
        else:
            len3 = len2

        for i in range(len3):
            if s1[i] != s2[i]:
                return i

        return len3


    def add(self, term, ti):
        # print 'tiw add:',term
        if not self.isIndex and term <= self.lastTerm:
            raise Exception, "term out of order: " + str(term) + str(self.lastTerm)
        if ti.freqPointer < self.lastTi.freqPointer:
            raise Exception, "freqPointer out of order"
        if ti.proxPointer < self.lastTi.proxPointer:
            raise Exception, "proxPointer out of order"

        if (not self.isIndex and self.size % self.INDEX_INTERVAL == 0):
            # add an index term
            self.other.add(self.lastTerm, self.lastTi)

        # write term
        self.writeTerm(term)
        # write doc freq
        self.output.writeVInt(ti.docFreq)
        # write pointers
        self.output.writeVLong(ti.freqPointer - self.lastTi.freqPointer)
        self.output.writeVLong(ti.proxPointer - self.lastTi.proxPointer)

        if self.isIndex:
            self.output.writeVLong(self.other.output.getFilePointer() - self.lastIndexPointer)
            self.lastIndexPointer = self.other.output.getFilePointer()

        self.lastTi.setTo(ti)
        self.size += 1


    def close(self):
        self.output.seek(0)
        self.output.writeInt(self.size)
        self.output.close()

        if self.isIndex is not True:
            self.other.close()


    def writeTerm(self, term):
        start = self.stringDifference(self.lastTerm.text(), term.text());
        length = len(term.text()) - start;

        # write shared prefix length
        self.output.writeVInt(start)
        # write delta length
        self.output.writeVInt(length)
        # write delta chars
        self.output.writeChars(term.text(), start, length)
        # write field num
        self.output.writeVInt(self.fieldInfos.fieldNumber(term.field()))
        self.lastTerm = term
