# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

__doc__ = """Superclass for L{ExactPhraseScorer} and L{SloppyPhraseScorer}."""

import lupy.util.priorityqueue as priorityqueue
import similarity
import phrasequeue
import scorer
import phrasepositions


class PhraseScorer(scorer.Scorer):
    slots = ['first', 'last', 'norms', 'pq', 'weight']
    
    
    def __init__(self, tps, n, w):
        self.norms = n
        self.weight = w
        
        # use PQ to build a sorted list of PhrasePositions
        self.pq = phrasequeue.PhraseQueue()
        
        i = 0
        for tp in tps:
            self.pq.put(phrasepositions.PhrasePositions(tp, i))
            i += 1
        self.pqToList()
        
        
    def firstToLast(self):
        # move first to end of list
        self.last.nxt = self.first
        self.last = self.first
        self.first = self.first.nxt
        self.last.nxt = None
        
        
    def phraseQuery(self):
        """Subclass responsibility"""
        
        
    def pqToList(self):
        self.last = None
        self.first = None
        
        while self.pq.top() is not None:
            pp = self.pq.pop()
            if self.last is not None:
                # add next to end of list
                self.last.nxt = pp
            else:
                self.first = pp
            self.last = pp
            pp.nxt = None
            
            
    def score(self, results, end):
        # find doc w/ all the terms
        while self.last.doc < end:
            # scan forward in first
            while self.first.doc < self.last.doc:
                self.first.next()
                while self.first.doc < self.last.doc:
                    self.first.next()
                self.firstToLast()
                if self.last.doc >= end:
                    return
                
            # found doc with all terms
            # check for phrase
            freq = self.phraseFreq()
            
            if freq > 0.0:
                # compute score
                score = similarity.tf(freq) * self.weight
                # normalize
                score *= similarity.normByte(self.norms[self.first.doc])
                # add to results
                results.collect(self.first.doc, score)
            # resume scanning
            self.last.next()
                
                
        
        
    
