# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

__doc__ = """/** The abstract base class for queries.
  Instantiable subclasses are:
  
  -L{TermQuery}
  -L{PhraseQuery}
  -L{BooleanQuery}
  
  """

import math

class Query(object):
    __slots__ = ['boost']

    # class methods
    def getScorer(cls, query, searcher, reader):
        query.prepare(reader)
        sum = query.sumOfSquaredWeights(searcher)
        norm = 1.0/math.sqrt(sum)
        query.normalize(norm)
        return query.scorer(reader)

    getScorer = classmethod(getScorer)
    

    # instance methods
    
    def __init__(self):
        self.boost = 1.0
        

    def getBoost(self):
        """Gets the boost for this term.  Documents containing
        this term will (in addition to the normal weightings) have their score
        multiplied by <code>b</code>.   The boost is 1.0 by default."""

        return self.boost


    def normalize(self, norm):
        pass


    def prepare(self, reader):
        pass

    def scorer(self, reader):
        """query evaluation"""
        
        pass

    def setBoost(self, b):
        """Sets the boost for this term to <code>b</code>.  Documents containing
        this term will (in addition to the normal weightings) have their score
        multiplied by <code>b</code>."""

        self.boost = b


    def sumOfSquaredWeights(self, searcher):
        """query weighting"""
        
        pass

    
    def toString(self, field):
        """Prints a query to a string, with <code>field</code> as the default field
        for terms.
        <p>The representation used is one that is readable by
        <a href="doc/lucene.queryParser.QueryParser.html">QueryParser</a>
        (although, if the query was created by the parser, the printed
        representation may not be exactly what was parsed)."""
        



    
