# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

__doc__ = """The abstract base class for search implementations.

          Subclasses implement search over a single index, over
          multiple indices, and over indices on remote servers."""

import hits

class Searcher(object):


    def close(self):
        """Frees resources associated with this Searcher."""
        pass


    def doc(self, i):
        """For use by L{HitCollector} implementations."""
        pass


    def docFreq(self, term):
        pass


    def maxDoc(self):
        pass


    def searchQuery(self, query):
        """Returns the documents matching B{query}."""
        return self.searchQueryFilter(query, None)


    def searchQueryFilter(self, query, filter):
        """Returns the documents matching B{query} and B{filter}."""
        return hits.Hits(self, query, filter)


    def searchQueryFilterHitColl(self, query, filter, results):
        """Lower-level search API.

        L{HitCollector#collect(int,float)} is called for every
        non-zero scoring document.

        Applications should only use this if they need <i>all</i> of the
        matching documents.  The high-level search API
        ({L{Searcher#search(Query)}) is usually more efficient, as it skips
        non-high-scoring hits.

        - B{query} to match documents
        - B{filter} if non-null, a bitset used to eliminate some documents
        - B{results} to receive hits
        """
        pass
    

    def searchQueryFilterInt(self, query, filter, n):
        pass


    def searchQueryHitColl(self, query, results):
        """Lower-level search API.

        L{HitCollector#collect(int,float)} is called for every non-zero
        scoring document.

        Applications should only use this if they need I{all} of the
        matching documents.  The high-level search API (I{Searcher#search(Query)})
        is usually more efficient, as it skips non-high-scoring hits.
        """
        self.searchQueryFilterHitColl(query, None, results)
        
