# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

__doc__ = """A Query that matches documents containing a term.
            This may be combined with other terms with a L{BooleanQuery}."""

import query
import termscorer
import similarity

class TermQuery(query.Query):
    __slots__ = ['idf', 'term', 'weight']

    def __init__(self, t):
        """Constructs a query for the term B{t}."""
        
        query.Query.__init__(self)
        
        self.term = t
        self.idf = 0.0
        self.weight = 0.0


    def normalize(self, norm):
        # normalize for query
        self.weight *= norm
        # factor from document
        self.weight *= self.idf
        
        
    def scorer(self, reader):
        termDocs = reader.termDocsTerm(self.term)
        if termDocs is None:
            return None
        
        return termscorer.TermScorer(termDocs,
                                     reader.normsField(self.term.field()),
                                     self.weight)
    

    def sumOfSquaredWeights(self, searcher):
        self.idf = similarity.idfTerm(self.term, searcher)
        self.weight = self.idf * self.boost
        # sqaure term weights
        return self.weight * self.weight


    def toString(self, field):
        """Prints a user-readable version of this query"""

        buffer = ''
        if not self.term.field() == field:
            buffer += self.term.field() + ':'

        buffer += self.term.text()

        if self.boost != 1.0:
            buffer += '^' + str(self.boost)

        return buffer
    
    
