# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.


from array import array
import copy


class InputStream(object):
    # __slots__ = ['buffer', 'chars', 'bufferStart', 'bufferLength',
    #             'bufferPosition', 'length']

    BUFFER_SIZE = 1024

    def __init__(self):
        self.buffer = None
        self.chars = None

        self.bufferStart = 0
        self.bufferLength = 0
        self.bufferPosition = 0
        
        
    def clone(self):
        clone = copy.copy(self)
        clone.isClone = True
        return clone


    def readByte(self):
        if self.bufferPosition >= self.bufferLength:
            self.refill()
        res = self.buffer[self.bufferPosition]
        self.bufferPosition += 1
        return res


    def readBytes(self, b, offset, len):
        if len < self.BUFFER_SIZE:
            for i in range(len):
                b[i + offset] = self.readByte()
        else:
            start = self.getFilePointer()
            self.seekInternal(start)
            self.readInternal(b, offset, len)

            self.bufferStart = start + len
            self.bufferPosition = 0
            self.bufferLength = 0


    def readInt(self):
        return (((self.readByte() & 0xFF) << 24) |
                ((self.readByte() & 0xFF) << 16) |
                ((self.readByte() & 0xFF) << 8)  |
                (self.readByte() & 0xFF))


    def readVInt(self):
        b = self.readByte()
        i = b & 0x7F

        shift = 7
        while b & 0x80 != 0:
            b = self.readByte()
            i |= (b & 0x7F) << shift
            shift += 7
        return i


    def readLong(self):
        return(self.readInt() << 32 | (self.readInt() & 0xFFFFFFFFL))
    

    def readVLong(self):
        b = self.readByte()
        i = b & 0x7F

        shift = 7
        while b & 0x80 != 0:
            b = self.readByte()
            i |= (b & 0x7FL) << shift
            shift += 7

        return i
    

    def readString(self):
        length = self.readVInt()

        if (self.chars is None or length > len(self.chars)):
            self.chars = [' '] * length
            
        self.readChars(self.chars, 0, length)

        #tmp = u''
        tmp = ''
        for c in self.chars[:length]:
            tmp += c
        return tmp

    

    def readChars(self, buffer, start, length):
        end = start + length
        
        for i in range(start, end):
            b = self.readByte()
            if (b & 0x80) == 0:
                #buffer[i] = unichr(b & 0x7F) 
                buffer[i] = chr(b & 0x7F) 
            elif (b & 0xE0) != 0xE0:
                tmpInt = ((b & 0x1F << 6)|(self.readByte() & 0x3F))
                buffer[i] = unichr(tmpInt)
            else:
                buffer[i] = unichr((((b & 0x0f) << 12) | 
                             ((self.readByte() & 0x3F) << 6) |
                             (self.readByte() & 0x3F)))
                

    def refill(self):
        start = self.bufferStart + self.bufferPosition
        end = start + self.BUFFER_SIZE

        if end > self.length:
            end = self.length

        self.bufferLength = end - start

        if self.bufferLength == 0:
            raise IOError, 'read past EOF'

        if self.buffer is None:
            self.buffer = array('B',[0x00]* self.BUFFER_SIZE)

        self.readInternal(self.buffer, 0, self.bufferLength)

        self.bufferStart = start
        self.bufferPosition = 0



    def readInternal(self, b, offset, length):
        raise NotImplementedError()


    def close(self):
        raise NotImplementedError()


    def getFilePointer(self):
        return self.bufferStart + self.bufferPosition


    def seek(self, pos):
        if pos >= self.bufferStart and pos < (self.bufferStart + self.bufferLength):
            self.bufferPosition = pos - self.bufferStart
        else:
            self.bufferStart = pos
            self.bufferPosition = 0
            self.bufferLength = 0
            self.seekInternal(pos)


    def seekInternal(self, pos):
        raise Exception, 'No code here!'


    def length(self):
        return self.length
    
        
