# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

from array import array

def byte(i):
    return i & 0xFF


def mint(i):
    """My int cast"""
    return i & 0xFFFFFFFF
        

class OutputStream(object):
    __slots__ = ['buffer', 'bufferStart', 'bufferPosition','length']

    BUFFER_SIZE = 1024

    def __init__(self):
        self.buffer = array('B',[0]*self.BUFFER_SIZE)
        self.bufferStart = 0
        self.bufferPosition = 0

    
    def writeByte(self, b):
        if self.bufferPosition >= self.BUFFER_SIZE:
            self.flush()

        self.buffer[self.bufferPosition] = b
        self.bufferPosition += 1


    def writeBytes(self, b, length):
        for i in xrange(length):
            self.writeByte(b[i])


    def writeInt(self, i):
        self.writeByte(byte(i >> 24))
        self.writeByte(byte(i >> 16))
        self.writeByte(byte(i >> 8))
        self.writeByte(byte(i))


    def writeVInt(self, i):
        while (i & ~0x7F) != 0:
            self.writeByte((i & 0x7F) | 0x80)
            i = i >> 7
        self.writeByte(i)


    def writeLong(self, i):
        self.writeInt(mint(i >> 32))
        self.writeInt(mint(i))


    def writeVLong(self, i):
        while (i & ~0x7F) != 0:
            self.writeByte((i & 0x7F) | 0x80)
            i = i >> 7
        self.writeByte(i)

        
    def writeString(self, s):
        length = len(s)
        self.writeVInt(length)
        self.writeChars(s, 0, length)


    def writeChars(self, s, start, length):
        end = start + length
        for i in xrange(start, end):
            code = ord(s[i])
            if code >= 0x01 and code <= 0x7F:
                self.writeByte(code)
            elif code > 0x80 and code <= 0x7FF or code == 0:
                self.writeByte(0xC0 | code >> 6)
                self.writeByte(0x80 | code & 0x3F)
            else:
                self.writeByte(0xE0 | (code >> 12))
                self.writeByte(0x80 | ((code >> 6) & 0x3F))
                self.writeByte(0x80 | (code & 0x3F))


    def flush(self):
        self.flushBuffer(self.buffer, self.bufferPosition)
        self.bufferStart += self.bufferPosition
        self.bufferPosition = 0


    def flushBuffer(self, b, len):
        """???"""


    def close(self):
        self.flush()


    def getFilePointer(self):
        return self.bufferStart + self.bufferPosition


    def seek(self, pos):
        self.flush()
        self.bufferStart = pos


    def length(self):
        raise NotImplementedError()
