# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

class PriorityQueue(object):
    __slots__ = ['list']

    def __init__(self, ignore = None):
        self.list = []
        

    def insort(self, x, lo=0, hi=None):
        """Insert item x in list a, and keep it sorted assuming a is sorted.

        If x is already in a, insert it to the right of the rightmost x.

        Optional args lo (default 0) and hi (default len(a)) bound the
        slice of a to be searched.
        """
        
        if hi is None:
            hi = len(self.list)
        while lo < hi:
            mid = (lo+hi)//2
            if self.lessThan(x, self.list[mid]): hi = mid
            else: lo = mid+1
        self.list.insert(lo, x)
    

    def clear(self):
        self.list = []


    def pop(self):
        if len(self) > 0:
            res = self.list[0]
            self.list = self.list[1:]
        else:
            res = None
        return res


    def put(self, element):
        self.insort(element)


    def size(self):
        return len(self.list)


    def top(self):
        if len(self) > 0:
            return self.list[0]
        else:
            return None


    def __repr__(self):
        return 'PQueue:' + str(self.list)
    
    
    def __len__(self):
        return len(self.list)
            


class IntegerQueue(PriorityQueue):
    def lessThan(self, a, b):
        return a < b

    
