# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""Replace default RAMDirectory with one based on cStringIOs.
"""

import time
import cStringIO as StringIO
from array import array

from inputstream import InputStream
from outputstream import OutputStream
from directory import Directory
import fsdirectory


class RAMDirectory(Directory):
    __slots__ = ['files']

    
    def __init__(self):
        self.files = {}

    
    def list(self):
        return self.files.keys()


    def fileExists(self, name):
        return (self.files.get(name, None) is not None)


    def fileModified(self, name):
        file = self.files[name]
        return file.lastModified
    

    def fileLength(self, name):
        file=self.files[name]
        return len(file)
    

    def deleteFile(self, name):
        del(self.files[name])


    def renameFile(self, name, newName):
        file = self.files[name]
        del(self.files[name])
        self.files[newName]=file


    def createFile(self, name):
        file = StringIO.StringIO()
        self.files[name]=file
        return RAMOutputStream(file)


    def openFile(self, name):
        file = self.files[name]
        return RAMInputStream(StringIO.StringIO(file.getvalue()))


    def makeLock(self, name):
        """TBC"""


    def close(self):
        """Do nothing"""



class RAMInputStream(fsdirectory.FSInputStream):

    #__slots__ = ['file', 'pointer', 'length', 'isClone']

    def __init__(self, f):
        InputStream.__init__(self)
        
        self.isClone = False
        self.pointer = 0
        self.file = f
        self.length = len(self.file.getvalue())
        
        
    def readInternal(self, b, offset, length):
        position = self.getFilePointer()
        self.file.seek(position)

        total = 0
        while total < length:
            bytes = self.file.read(length-total)

            lb = len(bytes)

            b[0:lb]=array('B',bytes)
            
            total += lb
        
        
    def close(self):
        """Do nothing"""



class RAMOutputStream(fsdirectory.FSOutputStream):
    __slots__ = ['file', 'position']

    def __init__(self, f):
        OutputStream.__init__(self)
        
        self.file = f
        self.position = 0
        

    def length(self):
        return len(self.file.getvalue())
        

    def close(self):
        OutputStream.close(self)
        
        
class RAMDescriptor(object):
    __slots__ = ['position']
    
    
    def __init__(self):
        self.position = 0
