# This module is part of the Lupy project and is Copyright 2003 Amir Bakhtiar (amir@divmod.org). 
# This is free software; you can redistribute it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

import unittest

from lupy.store import inputstream, outputstream

class clsInputStream(inputstream.InputStream):

    def __init__(self, test_seq):
        inputstream.InputStream.__init__(self)
        self.UnitTestSeq = test_seq
        self.UnitTestLastPos = 0
        
    def readInternal(self, b, offset, length):
        start = offset + self.UnitTestLastPos
        end = min(start + length, len(self.UnitTestSeq))
        
        if start >= len(self.UnitTestSeq):
            self.lenght = 0
            return

        self.length = end - start
        
        j = 0
        for i in range(start, end):
            b[j] = self.UnitTestSeq[i]
            j+=1
            
        #print 'bufflen', self.length
        self.UnitTestLastPos += self.length
        

    def seekInternal(self, pos):
        self.UnitTestLastPos = pos


class clsOutputStream(outputstream.OutputStream):

    def __init__(self):
        outputstream.OutputStream.__init__(self)
        self.UnitTestString = [] # a list of bytes
        
    def flushBuffer(self, b, len):
        for i in xrange(0,len):
            self.UnitTestString.append(b[i])

    def getByteSeq(self):
        return self.UnitTestString


class TestCase(unittest.TestCase):

    def setUp(self):
        self.InVector1 = [5846,21,0,-56,25468,5,458,888]
        self.InVector2 = [u'martin',u'nil',u'SRA', u'',u'','Ziv',u'']
        self.InVector3 = []


    def testDigits(self):
        Out = clsOutputStream()
        for element in self.InVector1:
            Out.writeInt(element)
            Out.flush()
            
        tmpSeq = Out.getByteSeq()

        In = clsInputStream(tmpSeq)
        OutVector = [In.readInt() for i in self.InVector1]

        assert self.InVector1 == OutVector

    def testUnicode(self):
        Out = clsOutputStream()
        for element in self.InVector2:
            Out.writeString(element)
            Out.flush()
            
        tmpSeq = Out.getByteSeq()

        In = clsInputStream(tmpSeq)
        OutVector = [In.readString() for i in self.InVector2]

        assert self.InVector2 == OutVector

        
    def testEmpty(self):
        Out = clsOutputStream()
        for element in self.InVector3:
            Out.writeString(element)
            Out.flush()
            
        tmpSeq = Out.getByteSeq()

        In = clsInputStream(tmpSeq)
        OutVector = [In.readString() for i in self.InVector3]
            
        assert self.InVector3 == OutVector


#...one can play with the commented lines below to see what inputstream.py and outputstream.py returns
##print [a.encode('iso-8859-1', 'replace') for a in InVector], [a.encode('iso-8859-1', 'replace') for a in OutVector]
##print InVector, OutVector
##assert InVector == OutVector

        
def suite():
    TestCases = unittest.makeSuite(TestCase,'test')

    TestSuite = unittest.TestSuite()
    TestSuite.addTest(TestCases)
    
    return TestSuite



runner = unittest.TextTestRunner()
runner.run(suite())
