# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""
Test cases for lupy directory classes.
"""

import sys, os, time, string
import unittest
import cStringIO as StringIO

import lupy.store.ramdirectory as ramdirectory
import lupy.store.fsdirectory as fsdirectory

import lupy.index.indexwriter as indexwriter	
import lupy.index.indexreader as indexreader
from lupy.index.term import Term

import lupy.search.indexsearcher as indexsearcher
from lupy.search.termquery import TermQuery
from lupy.search.phrasequery import PhraseQuery

import lupy.document.document as document
import lupy.document.field as field

import lupy.analysis.simpleanalyzer

tempindexlocation='kai_test'

# Something big
dictionaryfile = 'words'

class QueryTestCase:
    """ Sets up an index with the a dictionary to search and run test cases on """

    def __init__(self, unicodetest = 0, num = 20):
        self.unicodetest = unicodetest
        self.num = num
        dictionary = open(dictionaryfile)
        self.docs = dictionary.readlines()[0:self.num]
        dictionary.close()

        print '*'*20
        if self.unicodetest:
            print "Testing unicode"
        else:
            print "Testing string"
        print '*'*20

        self.index()
        self.testTermSearch()
 
    def index(self):

        a = lupy.analysis.simpleanalyzer.SimpleAnalyzer()

        # Starts a new index
        iw = indexwriter.IndexWriter(tempindexlocation, a, True)
        
        key = 1

        for eachdoc in self.docs:
        
            if self.unicodetest:
                eachdoc = unicode(eachdoc)

            d = document.Document()
            f = field.Keyword('id', str(key))
            d.add(f)

            f = field.Text('text', eachdoc)
            d.add(f)

            print "Indexed", repr(eachdoc), key, type(eachdoc)
            iw.addDocument(d)
            key = key + 1
    
        iw.optimize()
        iw.close()

        self.searcher = indexsearcher.IndexSearcher(tempindexlocation)

    def termSearch(self, qStr):
        t = Term('text', qStr)
        q = TermQuery(t)
        
        return q

    def printHits(self, hits):
        if len(hits) == 0:
            print 'Nothing found!'
        else:
            for i in range(len(hits)):
                print hits.doc(i)
        

    def runQuery(self, q, searcher):
        """The run a query through a searcher and return the hits"""

        print 'Query:', q.toString('text')
        hits = searcher.searchQuery(q)
        print hits
        self.printHits(hits)
        print
        return hits


    def testTermSearch(self):

        dictionary = open(dictionaryfile)
        words = dictionary.readlines()[0:self.num]
        import random
        word = random.choice(words)
        word = word.strip()
        word = word.lower()

        # Unicode!
        if self.unicodetest:
            word = unicode(word)

        # Clean up
        dictionary.close()
        words = None

        q = self.termSearch(word)
        self.runQuery(q, self.searcher)

        print "Searching for:", repr(word), type(word)
        print

        t = Term('text', word)
        ph = TermQuery(t)
        
        hits = self.searcher.searchQuery(ph)
    
        print hits

        if hits.length != 1:
            print 'incorrect number of search results'
        self.searcher.close()
        #self.assertEquals(hits.length, 1)
        #print hits.doc(0)
        #print repr(hits.doc(0).getField('text').stringValue())
        #assert hits.doc(0).getField('text').stringValue() == word, 'incorrect text string returned'


if __name__ == "__main__":
    #test1 = QueryTestCase(0, 50)
    test1 = QueryTestCase(0, 20)
