# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""
Test cases for lupy directory classes.
"""

import sys, os, time
import unittest
import cStringIO as StringIO

import lupy.store.ramdirectory as ramdirectory
import lupy.store.fsdirectory as fsdirectory

import lupy.index.indexwriter as indexwriter	
import lupy.index.indexreader as indexreader
import lupy.index.term as term
from lupy.indexer import Index

import lupy.document.document as document
import lupy.document.field as field

import lupy.analysis.simpleanalyzer

import cStringIO as StringIO

INDEXNAME = 'lupytestindex'

class IndexWriterTestCase(unittest.TestCase):

    def testTerm(self):
        t = term.Term('atext', 'that')
        tt = term.Term('btext', 'those')
        ttt = term.Term('atext', 'that')
        self.failUnlessEqual(t > tt, False)

        l=[tt,t,tt]
        l.sort()
        self.failUnlessEqual(l, [t,tt,tt])

        d = {}
        d[t]=1
        self.failUnlessEqual(1, d[ttt])
        
        t = term.Term('this', 'apple')
        tt = term.Term('this', 'orange')
        self.failUnlessEqual(t > tt, False)

    def testFieldName(self):
        i=Index('tstind', True)
        i.index(_uid='1', subject='foo', sender='amir', text='this is it')
        self.failUnlessEqual(i.find('this')[0].get('uid'), '1')
        
        
    def getDoc(self):
        d = document.Document()
        f = field.Text('path','file')
        d.add(f)
        f = field.Keyword('modified','monday')
        d.add(f)
        return d
        
        
    def testIndexWriter(self):
        t = time.time()
        a = lupy.analysis.simpleanalyzer.SimpleAnalyzer()
        iw = indexwriter.IndexWriter(INDEXNAME, a, True)

        d = self.getDoc()
        f = field.Text('text',StringIO.StringIO('the quick brown fox jumps over the lazy dog'))
        d.add(f)
        iw.addDocument(d)
        
        iw.optimize()
        iw.close()
        

    def testIndexReader(self):
        return
        ir = indexreader.IndexReader(INDEXNAME)
        ir.close()

    def testIndexer(self):
        I = Index('testind', True)
        wl = ['this', 'is', 'a', 'list', 'of', 'strings']
        for i, word in zip(range(len(wl)), wl):
            I.index(_uid=str(i), text=word)
            h = I.find(word)
            self.failUnlessEqual(h[0].get('uid'), str(i))
            I.delete(uid=str(i))
            h = I.find(word)
            self.failUnlessEqual(len(h), 0)
   

        
testCases = [IndexWriterTestCase]

if __name__ == "__main__":
    unittest.main()
