# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""
Test cases for lupy search.
"""

import unittest

from lupy.index.indexwriter import IndexWriter	
from lupy.index.term import Term
from lupy.search.indexsearcher import IndexSearcher
from lupy.search.termquery import TermQuery
from lupy.search.phrasequery import PhraseQuery
from lupy.document.document import Document
from lupy.analysis.simpleanalyzer import SimpleAnalyzer
import lupy.document.field as field

testdir = '_test_index'

class QueryTestCase(unittest.TestCase):
    def setUp(self):
        a = SimpleAnalyzer()
        iw = IndexWriter(testdir, a, True)

        docs = ['quick brown fox jumps low', 'slow blue fox jumps high',
                'every dog that jumps is a fox']
        
        for i in range(len(docs)):
            dd = docs[i]
            d = Document()
            f = field.Text('text', dd)
            d.add(f)
            iw.addDocument(d)
        iw.close()

    def testPhraseSearch(self):
        searcher = IndexSearcher(testdir)
        t = Term('text', 'blue')
        ph = PhraseQuery()
        ph.add(t)
        t = Term('text', 'fox')
        ph.add(t)
        hits = searcher.searchQuery(ph)
        self.failUnlessEqual(len(hits), 1)
        searcher.close()

    def testTermSearch(self):
        searcher = IndexSearcher(testdir)
        t = Term('text', 'slow')
        ph = TermQuery(t)
        hits = searcher.searchQuery(ph)
        self.failUnlessEqual(len(hits), 1)
        searcher.close()
        
        
testCases = [QueryTestCase]

if __name__ == "__main__":
    unittest.main()
