# This module is part of the Lupy project and is Copyright 2003 Amir
# Bakhtiar (amir@divmod.org). This is free software; you can redistribute
# it and/or modify it under the terms of version 2.1 of the GNU Lesser
# General Public License as published by the Free Software Foundation.

"""
Test cases for lupy directory classes.
"""


import sys, tempfile
import unittest
import lupy.store.ramdirectory as ramdirectory
import lupy.store.fsdirectory as fsdirectory


class DirTestCase(unittest.TestCase):
    
    text1 = """The quick brown fox wasn't quick enough and got eaten by the hounds."""
    text2 = """The hounds of doom are quick and prowl the nine circles of hell."""

    longtext = text1 * 100
    
    def testRAMDir(self):
        rd = ramdirectory.RAMDirectory()
        os = rd.createFile('foo')
        os.writeInt(191166)
        os.writeVInt(sys.maxint)
        os.writeLong(3396000123)
        os.writeVLong(3396000234123)
        os.writeString(self.text1)
        os.writeString(self.longtext)
        
        os.close()
        del(os)

        input = rd.openFile('foo')

        res = input.readInt()
        self.failUnlessEqual(res, 191166)
        
        res = input.readVInt()
        self.failUnlessEqual(res, sys.maxint)
        
        res = input.readLong()
        self.failUnlessEqual(res, 3396000123)
        
        res = input.readVLong()
        self.failUnlessEqual(res, 3396000234123)
        
        res = input.readString()
        self.failUnlessEqual(res, self.text1)
        
        res = input.readString()
        self.failUnlessEqual(res, self.longtext)

        input.close()


    def testFSDir(self):
        fd = fsdirectory.FSDirectory(tempfile.mktemp(), True)
        os = fd.createFile('foo')
        os.writeInt(191166)
        os.writeVInt(sys.maxint)
        os.writeLong(3396000123)
        os.writeVLong(3396000234123)
        os.writeString(self.text1)
        os.writeString(self.longtext)
        
        os.close()
        del(os)

        input = fd.openFile('foo')

        res = input.readInt()
        self.failUnlessEqual(res, 191166)
        
        res = input.readVInt()
        self.failUnlessEqual(res, sys.maxint)
        
        res = input.readLong()
        self.failUnlessEqual(res, 3396000123)
        
        res = input.readVLong()
        self.failUnlessEqual(res, 3396000234123)
        
        res = input.readString()
        self.failUnlessEqual(res, self.text1)
        
        res = input.readString()
        self.failUnlessEqual(res, self.longtext)

        input.close()
        

        
testCases = [DirTestCase]

if __name__ == "__main__":
    unittest.main()
